@@ -99,7 +99,7 @@ g_emblemed_icon_init (GEmblemedIcon *emblemed)
  *
  * Creates a new emblemed icon for @icon with the emblem @emblem.
  *
- * Returns: a new #GIcon
+ * Returns: (transfer full): a new #GIcon
  *
  * Since: 2.18
  **/
@@ -128,7 +128,7 @@ g_emblemed_icon_new (GIcon   *icon,
  *
  * Gets the main icon for @emblemed.
  *
- * Returns: a #GIcon that is owned by @emblemed
+ * Returns: (transfer full): a #GIcon that is owned by @emblemed
  *
  * Since: 2.18
  **/
@@ -146,7 +146,8 @@ g_emblemed_icon_get_icon (GEmblemedIcon *emblemed)
  *
  * Gets the list of emblems for the @icon.
  *
- * Returns: a #GList of #GEmblem <!-- -->s that is owned by @emblemed
+ * Returns: (element-type utf8) (transfer none): a #GList of #GEmblem <!-- -->s that
+ * is owned by @emblemed
  *
  * Since: 2.18
  **/
@@ -159,40 +160,6 @@ g_emblemed_icon_get_emblems (GEmblemedIcon *emblemed)
   return emblemed->emblems;
 }
 
-
-/**
- * g_emblemed_icon_add_emblem:
- * @emblemed: a #GEmblemedIcon
- * @emblem: a #GEmblem
- *
- * Adds @emblem to the #GList of #GEmblem <!-- -->s.
- *
- * Since: 2.18
- **/
-void 
-g_emblemed_icon_add_emblem (GEmblemedIcon *emblemed,
-                            GEmblem       *emblem)
-{
-  g_return_if_fail (G_IS_EMBLEMED_ICON (emblemed));
-  g_return_if_fail (G_IS_EMBLEM (emblem));
-
-  g_object_ref (emblem);
-  emblemed->emblems = g_list_append (emblemed->emblems, emblem);
-}
-
-static guint
-g_emblemed_icon_hash (GIcon *icon)
-{
-  GEmblemedIcon *emblemed = G_EMBLEMED_ICON (icon);
-  GList *list;
-  guint hash = g_icon_hash (emblemed->icon);
-
-  for (list = emblemed->emblems; list != NULL; list = list->next)
-    hash ^= g_icon_hash (G_ICON (list->data));
-
-  return hash;
-}
-
 static gint
 g_emblem_comp (GEmblem *a,
                GEmblem *b)
@@ -209,6 +176,40 @@ g_emblem_comp (GEmblem *a,
   return 1;
 }
 
+/**
+ * g_emblemed_icon_add_emblem:
+ * @emblemed: a #GEmblemedIcon
+ * @emblem: a #GEmblem
+ *
+ * Adds @emblem to the #GList of #GEmblem <!-- -->s.
+ *
+ * Since: 2.18
+ **/
+void 
+g_emblemed_icon_add_emblem (GEmblemedIcon *emblemed,
+                            GEmblem       *emblem)
+{
+  g_return_if_fail (G_IS_EMBLEMED_ICON (emblemed));
+  g_return_if_fail (G_IS_EMBLEM (emblem));
+
+  g_object_ref (emblem);
+  emblemed->emblems = g_list_insert_sorted (emblemed->emblems, emblem,
+                                            (GCompareFunc) g_emblem_comp);
+}
+
+static guint
+g_emblemed_icon_hash (GIcon *icon)
+{
+  GEmblemedIcon *emblemed = G_EMBLEMED_ICON (icon);
+  GList *list;
+  guint hash = g_icon_hash (emblemed->icon);
+
+  for (list = emblemed->emblems; list != NULL; list = list->next)
+    hash ^= g_icon_hash (G_ICON (list->data));
+
+  return hash;
+}
+
 static gboolean
 g_emblemed_icon_equal (GIcon *icon1,
                        GIcon *icon2)
@@ -223,9 +224,6 @@ g_emblemed_icon_equal (GIcon *icon1,
   list1 = emblemed1->emblems;
   list2 = emblemed2->emblems;
 
-  list1 = g_list_sort (list1, (GCompareFunc) g_emblem_comp);
-  list2 = g_list_sort (list2, (GCompareFunc) g_emblem_comp);
-
   while (list1 && list2)
   {
     if (!g_icon_equal (G_ICON (list1->data), G_ICON (list2->data)))
