@@ -29,7 +29,6 @@
 #include <glib-object.h>
 #include "glibintl.h"
 
-#include "gioalias.h"
 
 /**
  * SECTION:gfileattribute
@@ -165,6 +164,7 @@
  * <row><entry>%G_FILE_ATTRIBUTE_MOUNTABLE_CAN_UNMOUNT</entry><entry>mountable::can-unmount</entry><entry>boolean</entry></row>
  * <row><entry>%G_FILE_ATTRIBUTE_MOUNTABLE_CAN_EJECT</entry><entry>mountable::can-eject</entry><entry>boolean</entry></row>
  * <row><entry>%G_FILE_ATTRIBUTE_MOUNTABLE_UNIX_DEVICE</entry><entry>mountable::unix-device</entry><entry>uint32</entry></row>
+ * <row><entry>%G_FILE_ATTRIBUTE_MOUNTABLE_UNIX_DEVICE_FILE</entry><entry>mountable::unix-device-file</entry><entry>string</entry></row>
  * <row><entry>%G_FILE_ATTRIBUTE_MOUNTABLE_HAL_UDI</entry><entry>mountable::hal-udi</entry><entry>string</entry></row>
  * <row><entry>%G_FILE_ATTRIBUTE_TIME_MODIFIED</entry><entry>time::modified</entry><entry>uint64</entry></row>
  * <row><entry>%G_FILE_ATTRIBUTE_TIME_MODIFIED_USEC</entry><entry>time::modified-usec</entry><entry>uint32</entry></row>
@@ -336,23 +336,9 @@ _g_file_attribute_value_dup (const GFileAttributeValue *other)
   return attr;
 }
 
-GType
-g_file_attribute_info_list_get_type (void)
-{
-  static volatile gsize g_define_type_id__volatile = 0;
-
-  if (g_once_init_enter (&g_define_type_id__volatile))
-    {
-      GType g_define_type_id =
-        g_boxed_type_register_static (I_("GFileAttributeInfoList"),
-                                      (GBoxedCopyFunc) g_file_attribute_info_list_dup,
-                                      (GBoxedFreeFunc) g_file_attribute_info_list_unref);
-
-      g_once_init_leave (&g_define_type_id__volatile, g_define_type_id);
-    }
-
-  return g_define_type_id__volatile;
-}
+G_DEFINE_BOXED_TYPE (GFileAttributeInfoList, g_file_attribute_info_list,
+                     g_file_attribute_info_list_dup,
+                     g_file_attribute_info_list_unref)
 
 static gboolean
 valid_char (char c)
@@ -1067,6 +1053,3 @@ g_file_attribute_info_list_add (GFileAttributeInfoList *list,
 
   list_update_public (priv);
 }
-
-#define __G_FILE_ATTRIBUTE_C__
-#include "gioaliasdef.c"
