@@ -30,7 +30,6 @@
 #include "gioerror.h"
 #include "glibintl.h"
 
-#include "gioalias.h"
 
 /**
  * SECTION:gfileenumerator
@@ -186,7 +185,7 @@ g_file_enumerator_init (GFileEnumerator *enumerator)
  * enumerator is at the end, %NULL will be returned and @error will
  * be unset.
  *
- * Return value: A #GFileInfo or %NULL on error or end of enumerator.
+ * Return value: (transfer full): A #GFileInfo or %NULL on error or end of enumerator.
  *    Free the returned object with g_object_unref() when no longer needed.
  **/
 GFileInfo *
@@ -391,7 +390,7 @@ g_file_enumerator_next_files_async (GFileEnumerator     *enumerator,
  * 
  * Finishes the asynchronous operation started with g_file_enumerator_next_files_async().
  * 
- * Returns: a #GList of #GFileInfo<!---->s. You must free the list with 
+ * Returns: (transfer full) (element-type Gio.FileInfo): a #GList of #GFileInfo<!---->s. You must free the list with 
  *     g_list_free() and unref the infos with g_object_unref() when you're 
  *     done with them.
  **/
@@ -587,7 +586,7 @@ g_file_enumerator_set_pending (GFileEnumerator *enumerator,
  *
  * Get the #GFile container which is being enumerated.
  *
- * Returns: the #GFile which is being enumerated.
+ * Returns: (transfer full): the #GFile which is being enumerated.
  *
  * Since: 2.18
  */
@@ -759,6 +758,3 @@ g_file_enumerator_real_close_finish (GFileEnumerator  *enumerator,
 	    g_file_enumerator_real_close_async);
   return TRUE;
 }
-
-#define __G_FILE_ENUMERATOR_C__
-#include "gioaliasdef.c"
