@@ -1137,6 +1137,7 @@ g_credentials_type_get_type (void)
       static const GEnumValue values[] = {
         { G_CREDENTIALS_TYPE_INVALID, "G_CREDENTIALS_TYPE_INVALID", "invalid" },
         { G_CREDENTIALS_TYPE_LINUX_UCRED, "G_CREDENTIALS_TYPE_LINUX_UCRED", "linux-ucred" },
+        { G_CREDENTIALS_TYPE_FREEBSD_CMSGCRED, "G_CREDENTIALS_TYPE_FREEBSD_CMSGCRED", "freebsd-cmsgcred" },
         { 0, NULL, NULL }
       };
       GType g_define_type_id =
@@ -1168,20 +1169,23 @@ g_dbus_message_byte_order_get_type (void)
 }
 
 GType
-g_dbus_message_filter_result_get_type (void)
+g_application_flags_get_type (void)
 {
   static volatile gsize g_define_type_id__volatile = 0;
 
   if (g_once_init_enter (&g_define_type_id__volatile))
     {
-      static const GEnumValue values[] = {
-        { G_DBUS_MESSAGE_FILTER_RESULT_NO_EFFECT, "G_DBUS_MESSAGE_FILTER_RESULT_NO_EFFECT", "no-effect" },
-        { G_DBUS_MESSAGE_FILTER_RESULT_MESSAGE_CONSUMED, "G_DBUS_MESSAGE_FILTER_RESULT_MESSAGE_CONSUMED", "message-consumed" },
-        { G_DBUS_MESSAGE_FILTER_RESULT_MESSAGE_ALTERED, "G_DBUS_MESSAGE_FILTER_RESULT_MESSAGE_ALTERED", "message-altered" },
+      static const GFlagsValue values[] = {
+        { G_APPLICATION_FLAGS_NONE, "G_APPLICATION_FLAGS_NONE", "flags-none" },
+        { G_APPLICATION_IS_SERVICE, "G_APPLICATION_IS_SERVICE", "is-service" },
+        { G_APPLICATION_IS_LAUNCHER, "G_APPLICATION_IS_LAUNCHER", "is-launcher" },
+        { G_APPLICATION_HANDLES_OPEN, "G_APPLICATION_HANDLES_OPEN", "handles-open" },
+        { G_APPLICATION_HANDLES_COMMAND_LINE, "G_APPLICATION_HANDLES_COMMAND_LINE", "handles-command-line" },
+        { G_APPLICATION_SEND_ENVIRONMENT, "G_APPLICATION_SEND_ENVIRONMENT", "send-environment" },
         { 0, NULL, NULL }
       };
       GType g_define_type_id =
-        g_enum_register_static (g_intern_static_string ("GDBusMessageFilterResult"), values);
+        g_flags_register_static (g_intern_static_string ("GApplicationFlags"), values);
       g_once_init_leave (&g_define_type_id__volatile, g_define_type_id);
     }
 
