@@ -29,7 +29,6 @@
 #include "gio-marshal.h"
 #include "glibintl.h"
 
-#include "gioalias.h"
 
 /**
  * SECTION:gmountoperation
@@ -708,6 +707,3 @@ g_mount_operation_reply (GMountOperation *op,
   g_return_if_fail (G_IS_MOUNT_OPERATION (op));
   g_signal_emit (op, signals[REPLY], 0, result);
 }
-
-#define __G_MOUNT_OPERATION_C__
-#include "gioaliasdef.c"
