@@ -24,32 +24,35 @@
  * Modified by the GLib Team and others 1997-2000.  See the AUTHORS
  * file for a list of people on the GLib Team.  See the ChangeLog
  * files for a list of changes.  These files are distributed with
- * GLib at ftp://ftp.gtk.org/pub/gtk/. 
+ * GLib at ftp://ftp.gtk.org/pub/gtk/.
  */
 
-/* 
+/*
  * MT safe
  */
 
 #include "config.h"
 
-#include	<errno.h>
-#include	<stdlib.h>
-#include	<stdarg.h>
-#include	<string.h>
-#include	<stdio.h>
+#include <errno.h>
+#include <stdlib.h>
+#include <stdarg.h>
+#include <string.h>
+#include <stdio.h>
 #ifdef HAVE_UNISTD_H
-#include	<unistd.h>
+#include <unistd.h>
 #endif
 
-#include	"glib.h"
-#include	"gprintfint.h"
-#include        "galias.h"
-
 #ifdef G_OS_WIN32
-#include	<io.h>		/* For _read() */
+#include <io.h> /* For _read() */
 #endif
 
+#include "gscanner.h"
+
+#include "gprintfint.h"
+#include "gstrfuncs.h"
+#include "gstring.h"
+#include "gtestutils.h"
+
 /* --- defines --- */
 #define	to_lower(c)				( \
 	(guchar) (							\
@@ -744,9 +747,9 @@ g_scanner_sync_file_offset (GScanner *scanner)
   g_return_if_fail (scanner != NULL);
 
   /* for file input, rewind the filedescriptor to the current
-   * buffer position and blow the file read ahead buffer. usefull for
-   * third party uses of our filedescriptor, which hooks onto the current
-   * scanning position.
+   * buffer position and blow the file read ahead buffer. useful
+   * for third party uses of our file descriptor, which hooks 
+   * onto the current scanning position.
    */
 
   if (scanner->input_fd >= 0 && scanner->text_end > scanner->text)
@@ -1705,8 +1708,7 @@ g_scanner_get_token_ll	(GScanner	*scanner,
   
   if (gstring)
     {
-      value.v_string = gstring->str;
-      g_string_free (gstring, FALSE);
+      value.v_string = g_string_free (gstring, FALSE);
       gstring = NULL;
     }
   
@@ -1763,6 +1765,3 @@ g_scanner_get_token_ll	(GScanner	*scanner,
   *token_p = token;
   *value_p = value;
 }
-
-#define __G_SCANNER_C__
-#include "galiasdef.c"
